==> nhttpd-0.0.3.tar.gz: nhttpd-0.0.3/src/main.c <==
#include <header.h>

struct config_t		config;

struct mime_t		*mimes = NULL;
int			mimes_len = 0;

struct thread_t		*thread_pool;

time_t			now;

int			terminate = FALSE;


int main( int argc, char *argv[] )
{
	attr_server		**server = NULL;
	sock_addr		sa;
	sock_addr		client_sa;
	int			sz;
	struct pollfd		*listen_poll;
	int			poll_res;
	int			accept_fd;

	struct passwd		*pwd;
	uid_t			uid;
	gid_t			gid;

	struct linger		linger = { 1, LINGER_TIMEO };

	int			i;

	extern struct log_t	tlog;



	if ( parse_config( argc, argv, &config ) < 0 )
	{
		fprintf(stderr, "Config file parse error.");
		exit(0);
	}

	if ( terminate == TRUE )
		exit(0);

	if ( set_mime( &mimes, &mimes_len ) < 0 )
	{
		fprintf(stderr, "Mime file read error.\n");
		exit(1);
	}

	pwd = getpwnam( config.server_user );
	if ( pwd == (struct passwd*) 0 )
	{
		(void) fprintf( stderr, "unknown user - '%s'\n", config.server_user );
		exit( 1 );
	}
	uid = pwd->pw_uid;
	gid = pwd->pw_gid;

	thread_pool = malloc( sizeof(struct thread_t) * config.thread_pool_size );
	if ( thread_pool == (struct thread_t *) 0 )
	{
		fprintf(stderr, "Thread pool memory allocation failed.\n");
		exit(1);
	}

	server = malloc( config.port_len * sizeof( attr_server * ) );
	if ( server == NULL )
	{
		fprintf(stderr, "Server env memory allocation failed.\n");
		exit(0);
	}

	listen_poll = malloc( config.port_len * sizeof( struct pollfd ) );
	if ( listen_poll == NULL )
	{
		fprintf(stderr, "pollfd memory allocation failed.\n");
		exit(0);
	}

	for ( i=config.port_len; i--; )
	{
		server[i] = sock_init( &sa, config.port[i], config.listen_backlog );
		if ( server[i] == NULL )
		{
			fprintf(stderr, "socket initialize failed.\n");
			exit(0);
		}
		set_pollfd( &listen_poll[i], server[i]->listen_fd, POLLIN );
	}

#ifdef __RELEASE
	switch ( fork() )
	{
		case 0: break;
		case -1:
			fprintf(stderr, "fork() failed.\n");
			exit(1);
		default: exit(1);
	}

	(void) setsid();
	(void) fclose( stdin );
	(void) fclose( stdout );
	(void) fclose( stderr );
#endif
	(void) signal( SIGPIPE, SIG_IGN );

	if ( getuid() == 0 )
	{
		if ( setgroups( 0, (const gid_t*) 0 ) < 0 )
		{
			fprintf(stderr, "set groups error.");
			exit( 1 );
		}
		if ( setgid( gid ) < 0 )
		{
			fprintf(stderr, "set gid error.");
			exit( 1 );
		}
		if ( initgroups( config.server_user, gid ) < 0 )
			fprintf(stderr, "initgroups error.");

		if ( setuid( uid ) < 0 )
		{
			fprintf(stderr, "set uid error.");
			exit( 1 );
		}
	}

	/* tlog thread start */
	pthread_create( &(tlog.thread), NULL, tlog_proc, NULL );

	for ( i=0; i < config.thread_pool_size; i++ )
	{
		thread_pool[i].status = STAT_INIT;
		thread_pool[i].sockfd = -1;
		pthread_mutex_init( &(thread_pool[i].mutex), NULL );
		pthread_cond_init( &(thread_pool[i].cond), NULL );
		pthread_create( &(thread_pool[i].thread), NULL, proc, &(thread_pool[i]) );
	}

	sz = sizeof( client_sa );

	for ( ; ; )
	{
		time( &now );
		http_make_date( now );

		accept_fd = -1;
		poll_res = poll( listen_poll, config.port_len, 1000 );

		if ( poll_res == 0 )
			continue;

		for ( i=0; i < config.port_len && poll_res; i++ )
		{
			if ( listen_poll[i].revents & POLLIN )
			{
				accept_fd = accept( listen_poll[i].fd, &client_sa.sa, &sz );
				poll_res--;

				if ( accept_fd < 0 )
				{
					close( accept_fd );
					continue;
				}
				//printf("new client accepted\n");
#ifdef __NONBLOCK
				if ( fcntl( accept_fd, F_SETFL, O_RDWR|O_NONBLOCK|O_ASYNC|O_NDELAY ) < 0 )
				{
					close( accept_fd );
					continue;
				}
#endif
#ifdef __LINGER
				setsockopt( accept_fd, SOL_SOCKET, SO_LINGER, &linger, sizeof(linger));
#endif
				if ( throw_thread_pool( accept_fd ) < 0 )
				{
					close(accept_fd);
				}
			}
		}
	}

	return 0;
}


int throw_thread_pool( int accept_fd )
{
	int p,i;

	for ( p=THREADPOOL_CONTROL_LOOP; p--;)
	{
		for ( i=config.thread_pool_size; i--; )
		{
			if ( thread_pool[i].status == STAT_SLEEPING )
			{
				//DEBUG
				pthread_mutex_lock( &(thread_pool[i].mutex) );
				thread_pool[i].sockfd = accept_fd;
				thread_pool[i].status = STAT_WORKING;
				pthread_cond_signal( &(thread_pool[i].cond) );
				pthread_mutex_unlock( &(thread_pool[i].mutex) );
				return 0;
			}
		}
		usleep(500);
	}

	return -1;
}

==> nhttpd-0.1.1.tar.gz: nhttpd-0.1.1/src/main.c <==
#include <header.h>

struct config_t		config;

struct mime_t		*mimes = NULL;
int			mimes_len = 0;

struct thread_t		*thread_pool;

time_t			now;

int			terminate = FALSE;


int main( int argc, char *argv[] )
{
	attr_server		**server = NULL;
	sock_addr		sa;
	sock_addr		client_sa;
	int			sz;
	struct pollfd		*listen_poll;
	int			poll_res;
	int			accept_fd;

	struct passwd		*pwd;
	uid_t			uid;
	gid_t			gid;

	struct linger		linger = { 1, LINGER_TIMEO };

	int			i;

	extern struct log_t	tlog;


	if ( parse_config( argc, argv, &config ) < 0 )
	{
		fprintf(stderr, "*** Config file parse error.");
		exit(0);
	}

	if ( terminate == TRUE )
		exit(0);

	if ( set_mime( &mimes, &mimes_len ) < 0 )
	{
		fprintf(stderr, "*** Mime file read error.\n");
		exit(1);
	}

	pwd = getpwnam( config.server_user );
	if ( pwd == (struct passwd*) 0 )
	{
		fprintf( stderr, "*** unknown user - '%s'\n", config.server_user );
		exit( 1 );
	}
	uid = pwd->pw_uid;
	gid = pwd->pw_gid;

	thread_pool = malloc( sizeof(struct thread_t) * config.thread_pool_size );
	if ( thread_pool == (struct thread_t *) 0 )
	{
		fprintf(stderr, "*** Thread pool memory allocation failed.\n");
		exit(1);
	}

	server = malloc( config.port_len * sizeof( attr_server * ) );
	if ( server == NULL )
	{
		fprintf(stderr, "*** Server env memory allocation failed.\n");
		exit(0);
	}

	listen_poll = malloc( config.port_len * sizeof( struct pollfd ) );
	if ( listen_poll == NULL )
	{
		fprintf(stderr, "*** Pollfd memory allocation failed.\n");
		exit(0);
	}

	for ( i=config.port_len; i--; )
	{
		server[i] = sock_init( &sa, config.port[i], config.listen_backlog );
		if ( server[i] == NULL )
		{
			fprintf(stderr, "*** Socket initialize failed.\n");
			exit(0);
		}
		set_pollfd( &listen_poll[i], server[i]->listen_fd, POLLIN );
	}

#ifdef __RELEASE
	switch ( fork() )
	{
		case 0: break;
		case -1:
			fprintf(stderr, "*** Fork() failed.\n");
			exit(1);
		default: exit(1);
	}

	(void) setsid();
#endif

	if ( getuid() == 0 )
	{
		printf("*** Switch process owner: %s\n*** Check LOG FILE DIRECTORY(%s/) permision and user/group.\n", config.server_user, tlog.path );
		if ( setgroups( 0, (const gid_t*) 0 ) < 0 )
		{
			fprintf(stderr, "*** Set groups error.");
			exit( 1 );
		}
		if ( setgid( gid ) < 0 )
		{
			fprintf(stderr, "*** Set gid error.");
			exit( 1 );
		}
		if ( initgroups( config.server_user, gid ) < 0 )
			fprintf(stderr, "*** Initgroups error.");

		if ( setuid( uid ) < 0 )
		{
			fprintf(stderr, "*** Set uid error.");
			exit( 1 );
		}
	}

	fprintf(stderr, "--- nhttpd started.\n");

	time( &now );
	/* tlog thread start */
	tlog_init();
	pthread_create( &(tlog.thread), NULL, tlog_proc, NULL );


	(void) signal( SIGPIPE, SIG_IGN );
#ifdef __RELEASE
	(void) fclose( stdin );
	(void) fclose( stdout );
	(void) fclose( stderr );
#endif


	for ( i=config.thread_pool_size; i--; )
	{
		thread_pool[i].status = STAT_INIT;
		thread_pool[i].sockfd = -1;
		pthread_mutex_init( &(thread_pool[i].mutex), NULL );
		pthread_cond_init( &(thread_pool[i].cond), NULL );
		pthread_create( &(thread_pool[i].thread), NULL, proc, &(thread_pool[i]) );
	}

	fprintf(stderr, "*** %d working threads start.\n", config.thread_pool_size);

	sz = sizeof( client_sa );

	for ( ; ; )
	{
		time( &now );
		http_make_date( now );

		accept_fd = -1;
		poll_res = poll( listen_poll, config.port_len, 1000 );

		if ( poll_res == 0 )
			continue;

		for ( i=0; i < config.port_len && poll_res; i++ )
		{
			if ( listen_poll[i].revents & POLLIN )
			{
				accept_fd = accept( listen_poll[i].fd, &client_sa.sa, &sz );
				poll_res--;

				if ( accept_fd < 0 )
				{
					close( accept_fd );
					continue;
				}
				//printf("new client accepted\n");
#ifdef __NONBLOCK
				if ( fcntl( accept_fd, F_SETFL, O_RDWR|O_NONBLOCK|O_ASYNC|O_NDELAY ) < 0 )
				{
					close( accept_fd );
					continue;
				}
#endif
#ifdef __LINGER
				setsockopt( accept_fd, SOL_SOCKET, SO_LINGER, &linger, sizeof(linger));
#endif
				if ( throw_thread_pool( accept_fd, &(client_sa.sa_in) ) < 0 )
				{
					close(accept_fd);
				}
			}
		}
	}

	return 0;
}


int throw_thread_pool( int accept_fd, struct sockaddr_in *addr )
{
	int p,i;

	for ( p=THREADPOOL_CONTROL_LOOP; p--;)
	{
		for ( i=config.thread_pool_size; i--; )
		{
			if ( thread_pool[i].status == STAT_SLEEPING )
			{
				//DEBUG
				pthread_mutex_lock( &(thread_pool[i].mutex) );
				thread_pool[i].sockfd = accept_fd;
				memcpy( &(thread_pool[i].addr), addr, sizeof(struct sockaddr_in) );
				thread_pool[i].status = STAT_WORKING;
				pthread_cond_signal( &(thread_pool[i].cond) );
				pthread_mutex_unlock( &(thread_pool[i].mutex) );
				return 0;
			}
		}
		usleep(500);
	}

	return -1;
}
